#ifndef QTRANSFERFUNCTIONEDITOR2_H
#define QTRANSFERFUNCTIONEDITOR2_H

#include "ui_qtransferfunctioneditor2base.h"

namespace udg {

class QTransferFunctionView;
class TransferFunction;
class TransferFunctionEditor;

/**
    Nou widget d'edici de funcions de transferncia que ha de substituir els antics.
    Permet editar totalment una funci de transferncia: nom, color, opacitat scalar i opacitat del gradient.
    Permet desfer i refer els canvis, carregar i desar funcions, t una llista de funcions recents, etc.
    T una modalitat grfica i una de taula. Es pot crear amb noms un subconjunt de les caracterstiques, si no es volen les altres.
    Per exemple, es pot tenir un editor noms grfic i noms per opacitats scalars, sense res ms.
  */
class QTransferFunctionEditor2 : public QWidget, private ::Ui::QTransferFunctionEditor2Base {
Q_OBJECT
Q_ENUMS(View)

public:
    /// Caracterstiques de l'editor.
    enum Feature {
        Name = 0x01,
        Color = 0x02,
        ScalarOpacity = 0x04,
        GradientOpacity = 0x08,
        LoadSave = 0x10,
        Recent = 0x20,
        // 0x01 | 0x02 | 0x04 | 0x08 | 0x10 | 0x20
        Everything = 0x3f
    };
    Q_DECLARE_FLAGS(Features, Feature)

    /// Tipus de vistes de l'editor.
    enum View { Graphical, Table, Both };

public:
    /// Crea l'editor amb totes les caracterstiques i tots els tipus de vistes.
    explicit QTransferFunctionEditor2(QWidget *parent = 0);
    /// Crea l'editor amb les caracterstiques i les vistes passades per parmetre.
    explicit QTransferFunctionEditor2(Features features, View view, QWidget *parent = 0);

    /// Retorna la funci de transferncia representada actualment.
    const TransferFunction& transferFunction() const;
    /// Assigna la funci de transferncia.
    void setTransferFunction(const TransferFunction &transferFunction);

signals:
    /// S'emet quan canvia la funci de transferncia.
    void transferFunctionChanged(const TransferFunction &transferFunction);

private:
    Q_DISABLE_COPY(QTransferFunctionEditor2)

    /// Inicialitza l'editor amb els parmetres donats.
    void init(Features features, View view);
    /// Fa les connexions pertinents de signals i slots.
    void makeConnections();

private slots:
    /// Emet el signal transferFunctionChanged(TransferFunction).
    void emitTransferFunctionChanged();

private:
    /// L'autntic editor.
    TransferFunctionEditor *m_editor;
    /// Vista de la funci de transferncia.
    QTransferFunctionView *m_view;

};

} // End namespace udg

#endif // QTRANSFERFUNCTIONEDITOR2_H
